//**********************************************************************
// CRhsADOConnection
// =================
//
// Class to encapsulate an ADO Connection
//
// John Rennie
// jrennie@cix.compulink.co.uk
// 9th August 1998
//**********************************************************************

#ifndef _INC_CRHSADO
#define _INC_CRHSADO


//**********************************************************************
// Includes required by classes
// ----------------------------
//**********************************************************************

#include <adoint.h>
#include <adoid.h>
#include <Misc/RhsVariant.h>


//**********************************************************************
// Forward declarations
// --------------------
//**********************************************************************

class CRhsVariant;
class CRhsADOException;
class CRhsADOField;
class CRhsADORecordset;


//**********************************************************************
// CRhsADOConnection
// -----------------
//**********************************************************************

class CRhsADOConnection
{

  public:
    CRhsADOConnection();
    ~CRhsADOConnection();

    BOOL Open(const char* ConnectionString = NULL, const char* Username = NULL, const char* Password = NULL, int Options = 0);
    void Close(void);

    CRhsADORecordset& Execute(const char* Query, int Options, int* Affected = NULL);

    void ConnectionTimeout(LONG Timeout);
    LONG ConnectionTimeout(void);
    void CommandTimeout(LONG Timeout);
    LONG CommandTimeout(void);

    const char* GetLastError(char* Buffer, int MaxLen);

    inline ADOConnection* GetADOConnection(void) { return m_Connection; }

  protected:
    BOOL Instantiate(void);

  private:
    ADOConnection* m_Connection;
    HRESULT m_HR;
};

/*
get_ConnectionString([out] BSTR* pbstr)
put_ConnectionString([in] BSTR bstr)
get_CommandTimeout([out] LONG* plTimeout)
put_CommandTimeout([in] LONG lTimeout)
get_ConnectionTimeout([out] LONG* plTimeout)
put_ConnectionTimeout([in] LONG lTimeout)
get_Version([out] BSTR* pbstr)
BeginTrans([out] long* TransactionLevel)
CommitTrans(void)
RollbackTrans(void)
get_Errors([out] ADOErrors** ppvObject)
get_DefaultDatabase([out] BSTR* pbstr)
put_DefaultDatabase([in] BSTR bstr)
get_IsolationLevel([out] IsolationLevelEnum* Level)
put_IsolationLevel([in] IsolationLevelEnum Level)
get_Attributes([out] long* plAttr)
put_Attributes([in] long lAttr)
get_CursorLocation([out] CursorLocationEnum* plCursorLoc)
put_CursorLocation([in] CursorLocationEnum lCursorLoc)
get_Mode([out] ConnectModeEnum* plMode)
put_Mode([in] ConnectModeEnum lMode)
get_Provider([out] BSTR* pbstr)
put_Provider([in] BSTR Provider)
get_State([out] LONG* plObjState)
OpenSchema([in] SchemaEnum Schema, [in] VARIANT Restrictions, [in] VARIANT SchemaID, [out] _ADORecordset** pprset)
*/


//**********************************************************************
// CRhsADORecordset
// ----------------
//**********************************************************************

class CRhsADORecordset
{

  public:
    CRhsADORecordset();
    CRhsADORecordset(ADORecordset* Recordset);
    ~CRhsADORecordset();

    void Close(void);

    BOOL MoveNext(void);
    BOOL MovePrevious(void);
    BOOL MoveFirst(void);
    BOOL MoveLast(void);

    BOOL GetBOF(void);
    BOOL GetEOF(void);

    int FieldCount(void);
    CRhsADOField& GetField(int Number);
    CRhsADOField& GetField(const char* Name);

    CRhsADORecordset& operator=(CRhsADORecordset& Source);

    inline ADORecordset* GetADORecordset(void) { return m_Recordset; }

  private:
    ADORecordset* m_Recordset;
    ADOFields* m_Fields;
};

/*
get_Properties([out] ADOProperties** ppvObject);
get_AbsolutePosition([out] PositionEnum* pl);
put_AbsolutePosition([in] PositionEnum Position);
putref_ActiveADOConnection([in] IDispatch* pconn);
put_ActiveConnection([in] VARIANT vConn);
get_ActiveConnection([out] VARIANT* pvar);
get_Bookmark([out] VARIANT* pvBookmark);
put_Bookmark([in] VARIANT vBookmark);
get_CacheSize([out] long* pl);
put_CacheSize([in] long CacheSize);
get_CursorType([out] CursorTypeEnum* plCursorType);
put_CursorType([in] CursorTypeEnum lCursorType);
get_Fields([out] ADOFields** ppvObject);
get_LockType([out] LockTypeEnum* plLockType);
put_LockType([in] LockTypeEnum lLockType);
get_MaxRecords([out] long* plMaxRecords);
put_MaxRecords([in] long lMaxRecords);
get_RecordCount([out] long* pl);
putref_Source([in] IDispatch* pcmd);
put_Source([in] BSTR bstrConn);
get_Source([out] VARIANT* pvSource);
AddNew([in] VARIANT FieldList, [in] VARIANT Values);
CancelUpdate(_ADORecordset*  This);
Delete([in] AffectEnum AffectRecords);
GetRows([in] long Rows, [in] VARIANT Start, [in] VARIANT Fields, [out] VARIANT* pvar);
Move([in] long NumRecords, [in] VARIANT Start);
Open([in] VARIANT Source, [in] VARIANT ActiveConnection, [in] CursorTypeEnum CursorType, [in] LockTypeEnum LockType, [in] LONG Options);
Requery([in] LONG Options);
_xResync([in] AffectEnum AffectRecords);
Update([in] VARIANT Fields, [in] VARIANT Values);
get_AbsolutePage([out] PositionEnum* pl);
put_AbsolutePage([in] PositionEnum Page);
get_EditMode([out] EditModeEnum* pl);
get_Filter([out] VARIANT* Criteria);
put_Filter([in] VARIANT Criteria);
get_PageCount([out] long* pl);
get_PageSize([out] long* pl);
put_PageSize([in] long PageSize);
get_Sort([out] BSTR* Criteria);
put_Sort([in] BSTR Criteria);
get_Status([out] long* pl);
get_State([out] LONG* plObjState);
_xClone([out] _ADORecordset** ppvObject);
UpdateBatch([in] AffectEnum AffectRecords);
CancelBatch([in] AffectEnum AffectRecords);
get_CursorLocation([out] CursorLocationEnum* plCursorLoc);
put_CursorLocation([in] CursorLocationEnum lCursorLoc);
NextADORecordset([out] VARIANT* RecordsAffected, [out] _ADORecordset** ppiRs);
Supports([in] CursorOptionEnum CursorOptions, [out] VARIANT_BOOL* pb);
get_Collect([in] VARIANT Index, [out] VARIANT* pvar);
put_Collect([in] VARIANT Index, [in] VARIANT value);
get_MarshalOptions([out] MarshalOptionsEnum* peMarshal);
put_MarshalOptions([in] MarshalOptionsEnum eMarshal);
Find([in] BSTR Criteria, [in] LONG SkipRecords, [in] SearchDirectionEnum SearchDirection, [in] VARIANT Start);
Cancel(_ADORecordset*  This);
get_DataSource([out] IUnknown** ppunkDataSource);
putref_DataSource([in] IUnknown* punkDataSource);
Save([in] BSTR FileName, [in] PersistFormatEnum PersistFormat);
get_ActiveCommand([out] IDispatch** ppCmd);
put_StayInSync([in] VARIANT_BOOL bStayInSync);
get_StayInSync([out] VARIANT_BOOL* pbStayInSync);
GetString([in] StringFormatEnum StringFormat, [in] long NumRows, [in] BSTR ColumnDelimeter, [in] BSTR RowDelimeter, [in] BSTR NullExpr, [out] BSTR* pRetString);
get_DataMember([out] BSTR* pbstrDataMember);
put_DataMember([in] BSTR bstrDataMember);
CompareBookmarks([in] VARIANT Bookmark1, [in] VARIANT Bookmark2, [out] CompareEnum* pCompare);
Clone([in] LockTypeEnum LockType, [out] _ADORecordset** ppvObject);
Resync([in] AffectEnum AffectRecords, [in] ResyncEnum ResyncValues);
*/


//**********************************************************************
// CRhsADOField
// ------------
//**********************************************************************

class CRhsADOField
{

  public:
    CRhsADOField();
    CRhsADOField(ADOField* Field);
    ~CRhsADOField();

    void Close(void);

    char*        GetName(char* Name, int MaxLen);
    CRhsVariant& GetValue(CRhsVariant& Value);
    int          GetPrecision(void);
    int          GetType(void);

    CRhsADOField& operator=(CRhsADOField& Source);

    inline ADOField* GetADOField(void) { return m_Field; }

  private:
    ADOField* m_Field;
};

/*
get_Properties([out] ADOProperties** ppvObject);
get_ActualSize([out] long* pl);
get_Attributes([out] long* pl);
get_DefinedSize([out] long* pl);
put_Value([in] VARIANT Val);
get_NumericScale([out] BYTE* pbNumericScale);
AppendChunk([in] VARIANT Data);
GetChunk([in] long Length, [out] VARIANT* pvar);
get_OriginalValue([out] VARIANT* pvar);
get_UnderlyingValue([out] VARIANT* pvar);
get_DataFormat([out] IUnknown** ppiDF);
putref_DataFormat([in] IUnknown* piDF);
put_Precision([in] BYTE bPrecision);
put_NumericScale([in] BYTE bScale);
put_Type([in] DataTypeEnum DataType);
put_DefinedSize([in] long lSize);
put_Attributes([in] long lAttributes);
*/


//**********************************************************************
// CRhsADOException
// ----------------
//**********************************************************************

class CRhsADOException
{
  public:
    CRhsADOException(int ErrNum, const char* ErrStr);

    const char* GetError(CRhsADOConnection& Connection);

    int Err;
    char Error[256];
};


//**********************************************************************
// Helper functions
// ----------------
//**********************************************************************

BSTR SysAllocStringFromANSI(const char* s);


//**********************************************************************
// End of RhsADO.h
//**********************************************************************

#endif // _INC_CRHSADO

