//**********************************************************************
// CRhsADOField
// ============
//
// Class to encapsulate an ADO field
//
// John Rennie
// jrennie@cix.compulink.co.uk
// 15th November 1998
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include "RhsADO.h"


//**********************************************************************
// CRhsADOField::CRhsADOField
// CRhsADOField::~CRhsADOField
// ===========================
// Constructor and destructor
//**********************************************************************

CRhsADOField::CRhsADOField()
{
  m_Field = NULL;
}


CRhsADOField::CRhsADOField(ADOField* Field)
{
  m_Field = Field;
}


CRhsADOField::~CRhsADOField()
{
  if (m_Field)
    m_Field->Release();
}


//**********************************************************************
// CRhsADOField::Close
// -------------------
// Close the field and release the interface
//**********************************************************************

void CRhsADOField::Close(void)
{
  if (m_Field)
  { m_Field->Release();
    m_Field = NULL;
  }
}


//**********************************************************************
// CRhsADOField::GetName
// ---------------------
// Get a field's name
//**********************************************************************

char* CRhsADOField::GetName(char* Name, int MaxLen)
{ HRESULT hr;
  BSTR name;

  hr = m_Field->get_Name(&name);
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  WideCharToMultiByte(CP_ACP, 0, name, -1, Name, MaxLen, 0, NULL);

  return Name;
}


//**********************************************************************
// CRhsADOField::GetValue
// ----------------------
// Get a field's value as a CRhsVariant.
//**********************************************************************

CRhsVariant& CRhsADOField::GetValue(CRhsVariant& Value)
{ HRESULT hr;
  VARIANT value;

  hr = m_Field->get_Value(&value);
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  Value = value;

  return Value;
}


//**********************************************************************
// CRhsADOField::GetPrecision
// --------------------------
// Get a field's precision
//**********************************************************************

int CRhsADOField::GetPrecision(void)
{ HRESULT hr;
  BYTE precision;

  hr = m_Field->get_Precision(&precision);
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return (int) precision;
}


//**********************************************************************
// CRhsADOField::GetType
// ---------------------
// Get a field's type
//**********************************************************************

int CRhsADOField::GetType(void)
{ HRESULT hr;
  enum DataTypeEnum type;

  hr = m_Field->get_Type(&type);
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return (int) type;
}


//**********************************************************************
// CRhsADOField::operator=
// -----------------------
// The assignment operator is needed because CRhsADORecordset methods
// return a CRhsADOField&.
//**********************************************************************

CRhsADOField& CRhsADOField::operator=(CRhsADOField& Source)
{
  if (m_Field)
    m_Field->Release();

  m_Field = Source.m_Field;

  return *this;
}


