//**********************************************************************
// CRhsADORecordset
// =================
//
// Class to encapsulate an ADO recordset
//
// John Rennie
// jrennie@cix.compulink.co.uk
// 15th November 1998
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include "RhsADO.h"


//**********************************************************************
// CRhsADORecordset::CRhsADORecordset
// CRhsADORecordset::~CRhsADORecordset
// ===================================
// Constructor and destructor
//**********************************************************************

CRhsADORecordset::CRhsADORecordset()
{
  m_Recordset = NULL;
  m_Fields = NULL;
}


CRhsADORecordset::CRhsADORecordset(ADORecordset* Recordset)
{ HRESULT hr;

  m_Recordset = Recordset;

// We need to make sure the fields object is initialised

  hr = m_Recordset->get_Fields(&m_Fields);
}


CRhsADORecordset::~CRhsADORecordset()
{

// Release any interfaces we are holding on to

  if (m_Fields)
    m_Fields->Release();

  if (m_Recordset)
    m_Recordset->Release();
}


//**********************************************************************
// CRhsADORecordset::Close
// -----------------------
// Close the recordset and release the interface
//**********************************************************************

void CRhsADORecordset::Close(void)
{
  if (m_Fields)
  { m_Fields->Release();
    m_Fields = NULL;
  }

  if (m_Recordset)
  { m_Recordset->Close();
    m_Recordset->Release();
    m_Recordset = NULL;
  }
}


//**********************************************************************
// CRhsADORecordset::MoveNext
// CRhsADORecordset::MovePrevious
// CRhsADORecordset::MoveFirst
// CRhsADORecordset::MoveLast
// --------------------------
// Basic move commands
//**********************************************************************

BOOL CRhsADORecordset::MoveNext(void)
{ HRESULT hr;

  hr = m_Recordset->MoveNext();
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return TRUE;
}


BOOL CRhsADORecordset::MovePrevious(void)
{ HRESULT hr;

  hr = m_Recordset->MovePrevious();
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return TRUE;
}


BOOL CRhsADORecordset::MoveFirst(void)
{ HRESULT hr;

  hr = m_Recordset->MoveFirst();
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return TRUE;
}


BOOL CRhsADORecordset::MoveLast(void)
{ HRESULT hr;

  hr = m_Recordset->MoveLast();
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return TRUE;
}


//**********************************************************************
// CRhsADORecordset::BOF
// CRhsADORecordset::EOF
// ---------------------
// End/beginning of recordset flags
//**********************************************************************

BOOL CRhsADORecordset::GetBOF(void)
{ HRESULT hr;
  VARIANT_BOOL b;

  hr = m_Recordset->get_BOF(&b);
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return b;
}


BOOL CRhsADORecordset::GetEOF(void)
{ HRESULT hr;
  VARIANT_BOOL b;

  hr = m_Recordset->get_EOF(&b);
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return b;
}


//**********************************************************************
// CRhsADORecordset::FieldCount
// ----------------------------
// Return the number of fields per row.
//**********************************************************************

int CRhsADORecordset::FieldCount(void)
{ long l;
  HRESULT hr;

  hr = m_Fields->get_Count(&l);
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return l;
}


//**********************************************************************
// CRhsADORecordset::Field
// -----------------------
// Return a field from its ordinal number
//**********************************************************************

CRhsADOField& CRhsADORecordset::GetField(int Number)
{ HRESULT hr;
  CRhsVariant v;
  ADOField* field;
  CRhsADOField* rhs_field;

  v = Number;
  hr = m_Fields->get_Item((VARIANT) v, &field);
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  rhs_field = new CRhsADOField(field);

  return *rhs_field;
}


//**********************************************************************
// CRhsADORecordset::Field
// -----------------------
// Return a field from its name
//**********************************************************************

CRhsADOField& CRhsADORecordset::GetField(const char* Name)
{ HRESULT hr;
  CRhsVariant v;
  ADOField* field;
  CRhsADOField* rhs_field;

  v.AllocBSTR(Name);
  hr = m_Fields->get_Item((VARIANT) v, &field);
  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  rhs_field = new CRhsADOField(field);

  return *rhs_field;
}


//**********************************************************************
// CRhsADORecordset::operator=
// ---------------------------
// The assignment operator is needed because CRhsADOConnection methods
// return a CRhsADORecordset&.
//**********************************************************************

CRhsADORecordset& CRhsADORecordset::operator=(CRhsADORecordset& Source)
{
  if (m_Fields)
    m_Fields->Release();

  if (m_Recordset)
    m_Recordset->Release();

  m_Recordset = Source.m_Recordset;
  m_Fields = Source.m_Fields;

  return *this;
}


